module function_f
#(
	parameter DATA_WIDTH = 32
)
(
input      [DATA_WIDTH-1:0] 	  R0_i		    ,
input      [DATA_WIDTH-1:0] 	  R1_i		    ,
input      [DATA_WIDTH-1:0] 	  S0_i			,
input      [DATA_WIDTH-1:0]       S1_i			,
input      [DATA_WIDTH-1:0]		  K_key_even_i	,
input      [DATA_WIDTH-1:0]		  K_key_odd_i	,

output 	   [DATA_WIDTH-1:0]  	  F0_o		    ,
output 	   [DATA_WIDTH-1:0]  	  F1_o		    	
);

`include "clog_function.vh"

wire [DATA_WIDTH-1:0] h0_yield  ;
wire [DATA_WIDTH-1:0] h1_yield  ;
wire [DATA_WIDTH-1:0] pht_0  	;
wire [DATA_WIDTH-1:0] pht_1  	;
wire [DATA_WIDTH-1:0] R1_rol8	;

assign R1_rol8 = {R1_i[23:0],R1_i[(DATA_WIDTH-1):24]};

h_compact 
#(
	.DATA_WIDTH(DATA_WIDTH)
)
h0_inst
(	
    .data_i			 (R0_i	    ),
	.key_schedule0_i (S0_i 		),
	.key_schedule1_i (S1_i		), 
	                 
    .data_o			 (h0_yield	)
);

h_compact 
#(
	.DATA_WIDTH(DATA_WIDTH)
)
h1_inst
(	
    .data_i			 (R1_rol8	),
	.key_schedule0_i (S0_i 		),
	.key_schedule1_i (S1_i		), 
	                 
    .data_o			 (h1_yield	)
);

pht 
#(
	.DATA_WIDTH(DATA_WIDTH)
)
pht_inst
(
    .T0_i	 (h0_yield),
    .T1_i	 (h1_yield),
    .pht_o0  (pht_0	  ),
    .pht_o1  (pht_1	  )
);

assign  F0_o = (pht_0 + K_key_even_i);
assign  F1_o = (pht_1 + K_key_odd_i);

endmodule
