module key_generator
#(
	parameter DATA_WIDTH = 32,
	parameter COUNT_TO	 = 19
)
(
input 						      clk_i		   ,
input 						      reset_ni	   ,
input 							  keygen_en_i  ,
input      [(DATA_WIDTH*4)-1:0]   globalkey_i  ,

output reg [DATA_WIDTH-1:0]  	  K_key_odd_o  ,
output reg [DATA_WIDTH-1:0]  	  K_key_even_o	
);

`include "clog_function.vh"

wire [DATA_WIDTH-1 :0] key_index_concat32bits_even;
wire [DATA_WIDTH-1 :0] key_index_concat32bits_odd;

wire [clogb2(COUNT_TO):0] key_index;
wire [(DATA_WIDTH/4)-1:0] key_index_mul2;
wire [(DATA_WIDTH/4)-1:0] key_index_mul2add1;
wire [1:0] key_index_sample;
wire key_index_toggle;

assign key_index_mul2 = 2*key_index;
assign key_index_mul2add1 = key_index_mul2 +1;
assign key_index_concat32bits_even = {4{key_index_mul2}};
assign key_index_concat32bits_odd  = {4{key_index_mul2add1}};
assign key_index_toggle = (key_index_sample == 'd1);

// reg [DATA_WIDTH-1:0] keys_odd_memory  [0:COUNT_TO-1];
reg [DATA_WIDTH-1:0] keys_even_memory [0:COUNT_TO-1];

wire [DATA_WIDTH-1:0] globalkey_upmost ;
wire [DATA_WIDTH-1:0] globalkey_upper  ;
wire [DATA_WIDTH-1:0] globalkey_lower  ;
wire [DATA_WIDTH-1:0] globalkey_lowmost;

wire [DATA_WIDTH-1:0] h0_keygen_yield     ;
wire [DATA_WIDTH-1:0] h1_keygen_yield     ;
wire [DATA_WIDTH-1:0] h1_keygen_yield_rol8;
wire [DATA_WIDTH-1:0] pht1_yield_rol9	  ;
wire [DATA_WIDTH-1:0] pht_keygen0;
wire [DATA_WIDTH-1:0] pht_keygen1;

assign {globalkey_upmost,globalkey_upper,globalkey_lower,globalkey_lowmost} = globalkey_i[(DATA_WIDTH*4)-1:0];
assign h1_keygen_yield_rol8 = {h1_keygen_yield[23:0],h1_keygen_yield[(DATA_WIDTH-1):24]};
assign pht1_yield_rol9 = {pht_keygen1[22:0],pht_keygen1[(DATA_WIDTH-1):23]};

counter
#(
    .COUNT_TO(2)
)
counter_keyindex_sample
(
    .clk_i      (clk_i             ),
    .reset_ni	(reset_ni          ),
    .count_en_i (keygen_en_i       ),
    .count_o    (key_index_sample  )
);

counter
#(
    .COUNT_TO(COUNT_TO)
)
counter_keyindex
(
    .clk_i      (clk_i                  ),
    .reset_ni	(reset_ni               ),
    .count_en_i (key_index_toggle       ),
    .count_o    (key_index              )
);

function_h
#(
	.DATA_WIDTH(DATA_WIDTH)
)
h0_keygenINST
(	
	.clk_i			 (clk_i	    	             ),
	.reset_ni		 (reset_ni  	             ),
    .data_i			 (key_index_concat32bits_even),
	.key_schedule0_i (globalkey_upper	         ),
	.key_schedule1_i (globalkey_lowmost	         ), 
	                                             
    .data_o			 (h0_keygen_yield	         )
);

function_h
#(
	.DATA_WIDTH(DATA_WIDTH)
)
h1_keygenINST
(	
	.clk_i			 (clk_i	    		         ),
	.reset_ni		 (reset_ni  		         ),
    .data_i			 ( key_index_concat32bits_odd),
	.key_schedule0_i (globalkey_upmost	         ),
	.key_schedule1_i (globalkey_lower            ), 
	                                     
    .data_o			 (h1_keygen_yield	         )
);

pht 
#(
	.DATA_WIDTH(DATA_WIDTH)
)
pht_inst
(
	.clk_i	 (clk_i	  		 		),
	.reset_ni(reset_ni		 		),
    .T0_i	 (h0_keygen_yield		),
    .T1_i	 (h1_keygen_yield_rol8  ),
    .pht_o0  (pht_keygen0	  		),
    .pht_o1  (pht_keygen1	  		)
);

odd_key_memory
#(
	.DATA_WIDTH(DATA_WIDTH),
	.DEPTH(COUNT_TO)
)
odd_key_memoryINST
(
	.clk_i(clk_i),
    .reset_ni(reset_ni),
	.keygen_en_i(keygen_en_i),
	.key_index(key_index),
	.odd_key_i(K_key_odd_o)
);

always @(posedge clk_i or negedge reset_ni) begin
	if(~reset_ni)		K_key_odd_o <= 'b0;
	else 
		if(keygen_en_i)
			K_key_odd_o <= pht1_yield_rol9;
end

// always @(posedge clk_i or negedge reset_ni) begin
		// if(keygen_en_i)
			// keys_odd_memory[key_index] <= K_key_odd_o;
// end

always @(posedge clk_i or negedge reset_ni) begin
	if(~reset_ni)
		K_key_even_o <= 'b0;
	else
		if(keygen_en_i)
			K_key_even_o <= pht_keygen0;
end
        
 always @(posedge clk_i or negedge reset_ni) begin  
		if(keygen_en_i)
			keys_even_memory[key_index] <= K_key_even_o;
end       

endmodule