module s_box
#(
	parameter DATA_WIDTH = 8,
	parameter Q_LEFT 	 = 0,
	parameter Q_MIDDLE   = 0,
	parameter Q_RIGHT    = 0
)
(
    input 				    clk_i	  ,
    input 				    reset_ni  ,
    input  [DATA_WIDTH-1:0] left_q_i  ,
    input  [DATA_WIDTH-1:0] middle_q_i,
    input  [DATA_WIDTH-1:0] right_q_i ,
    
    output [DATA_WIDTH-1:0] left_q_o  ,
    output [DATA_WIDTH-1:0] middle_q_o,
    output [DATA_WIDTH-1:0] right_q_o
);

wire [DATA_WIDTH-1:0] left_q_yield   ;
wire [DATA_WIDTH-1:0] middle_q_yield ;
wire [DATA_WIDTH-1:0] right_q_o_yield;

assign left_q_o   = left_q_yield   ;
assign middle_q_o = middle_q_yield ;
assign right_q_o  = right_q_o_yield;

q_permutation 
#(
    .DATA_WIDTH(DATA_WIDTH),
    .Q_NR	   (Q_LEFT    )
)
q_box_left
(
    .clk_i	   (clk_i	      ),
    .reset_ni  (reset_ni      ),
    .q_i	   (left_q_i      ),
	
    .q_o	   (left_q_yield  )
);

q_permutation 
#(
    .DATA_WIDTH(DATA_WIDTH),
    .Q_NR	   (Q_MIDDLE  )
)
q_box_middle
(
    .clk_i	   (clk_i         ),
    .reset_ni  (reset_ni      ),
    .q_i	   (middle_q_i    ),
	
    .q_o	   (middle_q_yield)
);

q_permutation
 #(
    .DATA_WIDTH(DATA_WIDTH),
    .Q_NR	   (Q_RIGHT	  )
)
q_box_right
(
    .clk_i	   (clk_i	        ),
    .reset_ni  (reset_ni        ),
    .q_i	   (right_q_i       ),
	
    .q_o	   (right_q_o_yield )
);

endmodule
