module s_box_combined
#(
  parameter DATA_WIDTH = 32
)
(
    input 							clk_i		   ,
    input 							reset_ni	   ,
    input      [DATA_WIDTH-1:0]     data_i		   ,
    input      [DATA_WIDTH-1:0]     key_schedule0_i,
    input      [DATA_WIDTH-1:0]     key_schedule1_i,
	
    output [(DATA_WIDTH/4)-1:0] box0_o		   ,
    output [(DATA_WIDTH/4)-1:0] box1_o		   ,
    output [(DATA_WIDTH/4)-1:0] box2_o		   ,
    output [(DATA_WIDTH/4)-1:0] box3_o
);
 
wire [DATA_WIDTH-1:0] xor1;
wire [DATA_WIDTH-1:0] xor2;

wire [DATA_WIDTH-1:0] left_concatenate  ;
wire [DATA_WIDTH-1:0] middle_concatenate;

wire [(DATA_WIDTH/4)-1:0] data_upmost ;
wire [(DATA_WIDTH/4)-1:0] data_upper  ;  
wire [(DATA_WIDTH/4)-1:0] data_lower  ;  
wire [(DATA_WIDTH/4)-1:0] data_lowmost;

assign {data_upmost,data_upper,data_lower,data_lowmost} = data_i[(DATA_WIDTH-1):0];

wire [(DATA_WIDTH/4)-1:0] xor1_upmost ;
wire [(DATA_WIDTH/4)-1:0] xor1_upper  ;
wire [(DATA_WIDTH/4)-1:0] xor1_lower  ;
wire [(DATA_WIDTH/4)-1:0] xor1_lowmost;

assign {xor1_upmost,xor1_upper,xor1_lower,xor1_lowmost} = xor1[(DATA_WIDTH-1):0];

wire [(DATA_WIDTH/4)-1:0] xor2_upmost ;
wire [(DATA_WIDTH/4)-1:0] xor2_upper  ;
wire [(DATA_WIDTH/4)-1:0] xor2_lower  ;
wire [(DATA_WIDTH/4)-1:0] xor2_lowmost;

assign {xor2_upmost,xor2_upper,xor2_lower,xor2_lowmost} = xor2[(DATA_WIDTH-1):0];

wire [(DATA_WIDTH/4)-1:0] left0_yield;  
wire [(DATA_WIDTH/4)-1:0] left1_yield;  
wire [(DATA_WIDTH/4)-1:0] left2_yield;  
wire [(DATA_WIDTH/4)-1:0] left3_yield;

assign left_concatenate = {left0_yield,left1_yield,left2_yield,left3_yield};

wire [(DATA_WIDTH/4)-1:0] middle0_yield;  
wire [(DATA_WIDTH/4)-1:0] middle1_yield;  
wire [(DATA_WIDTH/4)-1:0] middle2_yield;  
wire [(DATA_WIDTH/4)-1:0] middle3_yield;

assign middle_concatenate = {middle0_yield,middle1_yield,middle2_yield,middle3_yield};

wire [(DATA_WIDTH/4)-1:0] right0_yield;  
wire [(DATA_WIDTH/4)-1:0] right1_yield;  
wire [(DATA_WIDTH/4)-1:0] right2_yield;  
wire [(DATA_WIDTH/4)-1:0] right3_yield;
  
assign xor1 = left_concatenate   ^ key_schedule0_i;
assign xor2 = middle_concatenate ^ key_schedule1_i;
 
s_box 
#(
    .DATA_WIDTH(DATA_WIDTH/4),
    .Q_LEFT	   (     0	    ),
    .Q_MIDDLE  (     0	    ),
    .Q_RIGHT   (     1	    ) 
)
s_box0
(
	.clk_i		(clk_i		  ),
	.reset_ni	(reset_ni	  ),
    .left_q_i	(data_upmost  ),
    .middle_q_i (xor1_upmost  ),
    .right_q_i	(xor2_upmost  ),
	
    .left_q_o	(left0_yield  ),
    .middle_q_o (middle0_yield),
    .right_q_o	(right0_yield )
);

s_box 
#(
	.DATA_WIDTH(DATA_WIDTH/4),
    .Q_LEFT	   (     1	    ),
    .Q_MIDDLE  (     0	    ),
    .Q_RIGHT   (     0	    ) 
)
s_box1
(
	.clk_i		(clk_i		  ),
	.reset_ni	(reset_ni	  ),
    .left_q_i	(data_upper   ),
    .middle_q_i (xor1_upper   ),
    .right_q_i	(xor2_upper   ),
	
    .left_q_o	(left1_yield  ),
    .middle_q_o (middle1_yield),
    .right_q_o	(right1_yield )
);

s_box #(
	.DATA_WIDTH(DATA_WIDTH/4),
    .Q_LEFT	   (     0	    ),
    .Q_MIDDLE  (     1	    ),
    .Q_RIGHT   (     1	    ) 
)
s_box2(
	.clk_i		(clk_i		  ),
	.reset_ni	(reset_ni	  ),
    .left_q_i	(data_lower   ),
    .middle_q_i (xor1_lower   ),
    .right_q_i	(xor2_lower   ),
	
    .left_q_o	(left2_yield  ),
    .middle_q_o (middle2_yield),
    .right_q_o	(right2_yield )
);

s_box #(
	.DATA_WIDTH(DATA_WIDTH/4),
    .Q_LEFT	   (     1	    ),
    .Q_MIDDLE  (     1	    ),
    .Q_RIGHT   (     0	    ) 
)
s_box3(
	.clk_i		(clk_i		  ),
	.reset_ni	(reset_ni	  ),
    .left_q_i	(data_lowmost ),
    .middle_q_i (xor1_lowmost ),
    .right_q_i	(xor2_lowmost ),
	
    .left_q_o	(left3_yield  ),
    .middle_q_o (middle3_yield),
    .right_q_o	(right3_yield )
);


assign box0_o = right0_yield;
assign box1_o = right1_yield;
assign box2_o = right2_yield;
assign box3_o = right3_yield;

endmodule
