module twofish
#(
	parameter DATA_WIDTH = 128,
	parameter COUNT_TO   = 17 ,
	parameter ADDR_WIDTH = 39
)
(
	input 				         clk_i		 ,
	input 				         reset_ni	 ,
	input 				         enable_i	 ,
	input 				         key_NOTdata ,
	input      [DATA_WIDTH-1:0]  plain_text_i,
	input      [DATA_WIDTH-1:0]  global_key_i,
	
	output reg [DATA_WIDTH-1:0]  cryptotext_o,
	output 						 busy_o
);

`include "clog_function.vh"

reg encryption_toggle;

wire [clogb2(COUNT_TO):0]   round_number;
wire [DATA_WIDTH-1:0] 		round_yield ;
wire [(DATA_WIDTH/4)-1:0] 	C0			;  
wire [(DATA_WIDTH/4)-1:0] 	C1			;
wire [(DATA_WIDTH/4)-1:0] 	C2			;
wire [(DATA_WIDTH/4)-1:0] 	C3			;
wire [(DATA_WIDTH/4)-1:0] 	R0_w		;
wire [(DATA_WIDTH/4)-1:0] 	R1_w		;
wire [(DATA_WIDTH/4)-1:0] 	R2_w		;
wire [(DATA_WIDTH/4)-1:0] 	R3_w		;
wire [(DATA_WIDTH/4)-1:0]   K_key_odd   ;
wire [(DATA_WIDTH/4)-1:0]   K_key_even  ;

wire [(DATA_WIDTH/4)-1:0]	K_inwhitening0;
wire [(DATA_WIDTH/4)-1:0]   K_inwhitening1;
wire [(DATA_WIDTH/4)-1:0]   K_inwhitening2;
wire [(DATA_WIDTH/4)-1:0]   K_inwhitening3;

wire [(DATA_WIDTH/4)-1:0]   K_outwhitening0;
wire [(DATA_WIDTH/4)-1:0]   K_outwhitening1;
wire [(DATA_WIDTH/4)-1:0]   K_outwhitening2;
wire [(DATA_WIDTH/4)-1:0]   K_outwhitening3;

wire [(DATA_WIDTH/4)-1:0] 	odd_key ;
wire [(DATA_WIDTH/4)-1:0]   even_key;

always @(posedge clk_i or negedge clk_i) begin
	if(~reset_ni)
		encryption_toggle <= 'b0;
	else
		if(round_number == 'd4)
			encryption_toggle <= ~encryption_toggle;
end

assign K_inwhitening0 = (odd_key) ? (round_number == 'd0) : 'b0;
assign K_inwhitening1 = (odd_key) ? (round_number == 'd1) : 'b0;
assign K_inwhitening2 = (odd_key) ? (round_number == 'd2) : 'b0;
assign K_inwhitening3 = (odd_key) ? (round_number == 'd3) : 'b0;

assign K_outwhitening0 = (even_key) ? (round_number == 'd0) : 'b0;
assign K_outwhitening1 = (even_key) ? (round_number == 'd1) : 'b0;
assign K_outwhitening2 = (even_key) ? (round_number == 'd2) : 'b0;
assign K_outwhitening3 = (even_key) ? (round_number == 'd3) : 'b0;

input_whitening
#(
    .DATA_WIDTH(DATA_WIDTH)
)
in_whiteningINST
(
	.clk_i		 (clk_i		       ),
	.reset_ni	 (reset_ni         ),
	.enable_i	 (encryption_toggle),
	.plain_text_i(plain_text_i     ),
	.K0_i		 (K_inwhitening0   ),
	.K1_i		 (K_inwhitening1   ),
	.K2_i		 (K_inwhitening2   ),
	.K3_i		 (K_inwhitening3   ),
    .R0_o		 (R0_w		       ),
    .R1_o		 (R1_w		       ),
    .R2_o		 (R2_w		       ),
    .R3_o		 (R3_w		       )
);

twofish_round
#(
	.DATA_WIDTH(DATA_WIDTH/4),
	.COUNT_TO  (COUNT_TO	)
)
twofish_roundDUT
(
	.clk_i		   (clk_i			 ),
	.reset_ni	   (reset_ni		 ),
	.enable_i	   (enable_i		 ),
	.R0_i		   (R0_w			 ),
	.R1_i		   (R1_w			 ),
	.R2_i		   (R2_w			 ),
	.R3_i		   (R3_w			 ),
	.round_number_o(round_number	 ),
	.toggle_o	   (				 ),
	.round_done	   (busy_o			 ),
	.round_o	   (round_yield 	 )
);

output_whitening
#(
	.DATA_WIDTH(DATA_WIDTH)
)
output_whiteningINST
(
	.clk_i	 (clk_i		       ),
	.reset_ni(reset_ni	       ),
    .enable_i(encryption_toggle),
	.data_i  (round_yield      ),
	.K0_i	 (K_outwhitening0  ),
	.K1_i	 (K_outwhitening1  ),
	.K2_i	 (K_outwhitening2  ),
	.K3_i	 (K_outwhitening3  ),
	.R0_o	 (C0		 	   ),
	.R1_o	 (C1		 	   ),
	.R2_o	 (C2		 	   ),
	.R3_o	 (C3		 	   )
);

key_generator_compact
#
(
	.DATA_WIDTH(DATA_WIDTH/4),
	.COUNT_TO  (COUNT_TO + 2)
)
key_generator_compactINST
(
	.clk_i		 (clk_i		  ),	  
	.reset_ni	 (reset_ni	  ), 
    .keygen_en_i (enable_i    ),
    .global_key_i(global_key_i),  
    .K_key_odd_o ( 			  ),
    .K_key_even_o(			  )
);

odd_key_memory
#(
	.DATA_WIDTH(DATA_WIDTH/4),
	.DEPTH	   (COUNT_TO + 2)
)
odd_key_memoryINST
(
	.clk_i		(clk_i		 ),
    .reset_ni	(reset_ni	 ),
	.keygen_en_i(enable_i 	 ),
	.key_index	(round_number),
	.odd_key_i	(			 ),
	.odd_key_o  (odd_key     )
);

even_key_memory
#(
	.DATA_WIDTH(DATA_WIDTH/4 ),
	.DEPTH	   (COUNT_TO + 2)
)
even_key_memoryINST
(
	.clk_i		(clk_i		 ),
    .reset_ni	(reset_ni	 ),
	.keygen_en_i(enable_i 	 ),
	.key_index  (round_number),
	.even_key_i (			 ),
	.even_key_o (even_key    )
);

always @(posedge clk_i or negedge reset_ni)begin
	if(~reset_ni)
		cryptotext_o <= 'b0;
	else
		cryptotext_o <= {C0,C1,C2,C3};
end

endmodule