`timescale 1ns/1ps
module twofish_round 
#(
  parameter DATA_WIDTH 		= 32,
  parameter COUNT_TO   		= 17,
  parameter COUNT_TO_SAMPLE = 2
)
(
input 						      clk_i		    ,
input 						      reset_ni		,
input 						      enable_i		,
input      [DATA_WIDTH-1:0] 	  R0_i			,
input      [DATA_WIDTH-1:0] 	  R1_i			,
input      [DATA_WIDTH-1:0] 	  R2_i			,
input      [DATA_WIDTH-1:0] 	  R3_i			,

output     [clogb2(COUNT_TO):0]   round_number_o,
output reg [(DATA_WIDTH*4)-1:0]   round_o		,
output reg 						  toggle_o		,
output 							  round_done
);

`include "clog_function.vh"

wire [DATA_WIDTH-1:0] F0;
wire [DATA_WIDTH-1:0] F1;

reg  [DATA_WIDTH-1:0] C2;
reg  [DATA_WIDTH-1:0] C3;

reg  [DATA_WIDTH-1:0] R0;
reg  [DATA_WIDTH-1:0] R1;
reg  [DATA_WIDTH-1:0] R2;
reg  [DATA_WIDTH-1:0] R3;

reg toggle_ff		   ;
reg round_number_sample;

wire [clogb2(COUNT_TO_SAMPLE):0]   counter_sample;
wire [DATA_WIDTH-1:0] 			   R2_xor_F0	 ;

counter 
#(
    .COUNT_TO(COUNT_TO)
 )
round_number_inst
(
	.clk_i		(clk_i							),
	.reset_ni	(reset_ni						),
    .count_en_i (toggle_o && round_number_sample),
    .count_o	(round_number_o					)
);

counter
#(
    .COUNT_TO(COUNT_TO_SAMPLE)
)
sample_inst
(
	.clk_i		(clk_i		   ),
	.reset_ni	(reset_ni	   ),
    .count_en_i (toggle_o	   ),
    .count_o	(counter_sample)    
);

function_f
#(
	.DATA_WIDTH(DATA_WIDTH),
	.COUNT_TO  (COUNT_TO  )
)

f_INST
(
	.clk_i		   (clk_i		  ),
	.reset_ni	   (reset_ni	  ),
    .R0_i		   (R0			  ),
    .R1_i		   (R1			  ),
	.round_number_i(round_number_o),
	.S0_i          (32'b0	      ),
	.S1_i          (32'b0	      ),
    .F0_o		   (F0			  ),
    .F1_o		   (F1			  )
);

assign R2_xor_F0     = (toggle_o) ? (R2 ^ F0) : 'b0;
assign round_done    = (round_number_o == 'd15)    ;

always @(posedge clk_i or negedge reset_ni) begin
    if(~reset_ni)
        round_number_sample <= 'b0;
    else
        if(enable_i)
            round_number_sample <= 1'b1;
        else    
            if(~(counter_sample == 'd1))
                round_number_sample <= 1'b1;
            else
                round_number_sample <= 1'b0;
end



always @(posedge clk_i or negedge reset_ni) begin
    if(~reset_ni)
        toggle_ff <= 'b0;
    else
        toggle_ff <= toggle_o;
end

always @(posedge clk_i or negedge reset_ni) begin
    if(~reset_ni)
        toggle_o <= 'b0;
    else
        if(enable_i)
            toggle_o <= ~toggle_o;
        else
        if(round_done)
            toggle_o <= 'b0;
end

always @(posedge clk_i or negedge reset_ni)begin
    if(~reset_ni)
        R0 <= 'b0;
    else
        if(enable_i)
            R0 <= R0_i;
        else
            if(toggle_ff)
                R0 <= C2;
end

always @(posedge clk_i or negedge reset_ni)begin
    if(~reset_ni)
        R1 <= 'b0;
    else
        if(enable_i)
            R1 <= R1_i;
        else
            if(toggle_ff)
                R1 <= C3;
end

always @(posedge clk_i or negedge reset_ni)begin
    if(~reset_ni)
        R2 <= 'b0;
    else
        if(enable_i)
            R2 <= R2_i;
        else
            if(toggle_ff)
                R2 <= R0;
end

always @(posedge clk_i or negedge reset_ni)begin
    if(~reset_ni)
        R3 <= 'b0;
    else
        if(enable_i)
            R3 <= R3_i;
        else
            if(toggle_ff)
                R3 <= R1;
end


always @(posedge clk_i or negedge reset_ni) begin
    if(~reset_ni)
        C2 <= 'b0;
    else
        if(toggle_o)
            C2 = {R2_xor_F0[0],R2_xor_F0[DATA_WIDTH-1:1]};
end
     
always @(posedge clk_i or negedge reset_ni) begin
    if(~reset_ni)
        C3 <= 'b0;
    else
        if(toggle_o)
            C3 = ({R3[DATA_WIDTH-2:0],R3[DATA_WIDTH-1]} ^ F1); 
end

always @(posedge clk_i or negedge reset_ni) begin
    if(~reset_ni)
        round_o <= 'b0;
    else
        if(toggle_o && round_done) 
            round_o <= {C2,C3,R0,R1};
        else 
            round_o <= 'b0;
end  

endmodule  
